searchNodes=[{"doc":"Functions to aid processing of tagged tuples in pipelines. Examples iex&gt; 1 ...&gt; |&gt; Result . from ( ) # {:ok, 1} ...&gt; |&gt; Result . map ( &amp; &amp;1 * 2 ) # {:ok, 2} ...&gt; |&gt; Result . then ( &amp; { :ok , &amp;1 + 1 } ) # {:ok, 3} ...&gt; |&gt; Result . unwrap_or_else ( 0 ) 3 iex&gt; &quot;Oh no!&quot; ...&gt; |&gt; Result . from_error ( ) # {:error, &quot;Oh no!&quot;} ...&gt; |&gt; Result . map ( &amp; &amp;1 * 2 ) # {:error, &quot;Oh no!&quot;} ...&gt; |&gt; Result . then ( &amp; { :ok , &amp;1 + 1 } ) # {:error, &quot;Oh no!&quot;} ...&gt; |&gt; Result . unwrap_or_else ( 0 ) 0 iex&gt; { :ok , 1 } ...&gt; |&gt; Result . map ( fn ...&gt; 1 -&gt; nil ...&gt; x -&gt; x * 2 ...&gt; end ) # :none ...&gt; |&gt; Result . default ( 3 ) # {:ok, 3} ...&gt; |&gt; Result . unwrap_or_else ( 0 ) 3 iex&gt; { :ok , 2 } ...&gt; |&gt; Result . map ( fn ...&gt; 1 -&gt; nil ...&gt; x -&gt; x * 2 ...&gt; end ) # {:ok, 4} ...&gt; |&gt; Result . default ( 3 ) # {:ok, 4} ...&gt; |&gt; Result . unwrap_or_else ( 0 ) 4 iex&gt; { :error , &quot;Oh no!&quot; } ...&gt; |&gt; Result . map ( fn ...&gt; 1 -&gt; nil ...&gt; x -&gt; x * 2 ...&gt; end ) # {:error, &quot;Oh no!&quot;} ...&gt; |&gt; Result . default ( 3 ) # {:error, &quot;Oh no!&quot;} ...&gt; |&gt; Result . unwrap_or_else ( 0 ) 0 iex&gt; { :error , &quot;Oh no!&quot; } ...&gt; |&gt; Result . error_then ( fn &quot;Oh no!&quot; -&gt; :error end ) # :error ...&gt; |&gt; Result . or_else ( fn -&gt; { :ok , 0 } end ) # {:ok, 0} ...&gt; |&gt; Result . unwrap! ( ) 0 iex&gt; { :ok , 1 } ...&gt; |&gt; Result . error_then ( fn &quot;Oh no!&quot; -&gt; :error end ) # {:ok, 1} ...&gt; |&gt; Result . or_else ( fn -&gt; { :ok , 0 } end ) # {:ok, 1} ...&gt; |&gt; Result . unwrap! ( ) 1","ref":"OkThen.Result.html","title":"OkThen.Result","type":"module"},{"doc":"If result is tagged with :ok , passes the wrapped value into the provided function (if provided) and returns :none . If result is not tagged with :ok , result is returned as-is. Equivalent to tagged_consume(result, :ok, func_or_value) . See tagged_consume/3 . Examples iex&gt; :ok |&gt; Result . consume ( ) :none iex&gt; :ok |&gt; Result . consume ( fn -&gt; &quot;hello&quot; end ) :none iex&gt; :ok |&gt; Result . consume ( fn { } -&gt; &quot;hello&quot; end ) :none iex&gt; { :ok , 1 } |&gt; Result . consume ( fn 1 -&gt; &quot;hello&quot; end ) :none iex&gt; { :ok , 1 , 2 } |&gt; Result . consume ( fn 1 , 2 -&gt; &quot;hello&quot; end ) ** (ArgumentError) Value-mapping function must have arity between 0 and 1. iex&gt; { :ok , 1 , 2 } |&gt; Result . consume ( fn { 1 , 2 } -&gt; &quot;hello&quot; end ) :none iex&gt; :error |&gt; Result . consume ( fn { } -&gt; &quot;hello&quot; end ) :error iex&gt; { :error , 1 } |&gt; Result . consume ( fn 1 -&gt; &quot;hello&quot; end ) { :error , 1 } iex&gt; &quot;bare value&quot; |&gt; Result . consume ( ) &quot;bare value&quot;","ref":"OkThen.Result.html#consume/2","title":"OkThen.Result.consume/2","type":"function"},{"doc":"If result is tagged :none , returns func_or_value wrapped as an :ok result. Otherwise, returns result . If func_or_value is a function, the returned value is used as the new value. If the new value is nil , then the result will remain :none . Consider using none_then/2 if you don't want this behaviour. If result is not tagged :none , result is returned as-is. Equivalent to default_as(result, :ok, func_or_value) . See default_as/3 . Examples iex&gt; :none |&gt; Result . default ( &quot;hello&quot; ) { :ok , &quot;hello&quot; } iex&gt; :none |&gt; Result . default ( { } ) :ok iex&gt; :none |&gt; Result . default ( nil ) :none iex&gt; :none |&gt; Result . default ( fn -&gt; 1 end ) { :ok , 1 } iex&gt; :none |&gt; Result . default ( fn { } -&gt; 1 end ) { :ok , 1 } iex&gt; { :none , 1 } |&gt; Result . default ( &amp; &amp;1 ) { :ok , 1 } iex&gt; :ok |&gt; Result . default ( &quot;hello&quot; ) :ok iex&gt; { :ok , 1 } |&gt; Result . default ( &quot;hello&quot; ) { :ok , 1 } iex&gt; { :ok , 1 , 2 } |&gt; Result . default ( &quot;hello&quot; ) { :ok , 1 , 2 } iex&gt; { :anything , 1 } |&gt; Result . default ( &quot;hello&quot; ) { :anything , 1 } iex&gt; &quot;bare value&quot; |&gt; Result . default ( &quot;hello&quot; ) &quot;bare value&quot;","ref":"OkThen.Result.html#default/2","title":"OkThen.Result.default/2","type":"function"},{"doc":"If result is tagged :none , returns func_or_value wrapped as a result with the given tag . Otherwise, returns result . If func_or_value is a function, the returned value is used as the new value. If the new value is nil , then the result will remain :none . Consider using none_then/2 if you don't want this behaviour. If result is not tagged :none , result is returned as-is. Examples iex&gt; :none |&gt; Result . default_as ( :ok , &quot;hello&quot; ) { :ok , &quot;hello&quot; } iex&gt; :none |&gt; Result . default_as ( :error , { } ) :error iex&gt; :none |&gt; Result . default_as ( :something , nil ) :none iex&gt; :none |&gt; Result . default_as ( :ok , fn -&gt; 1 end ) { :ok , 1 } iex&gt; :none |&gt; Result . default_as ( :ok , fn { } -&gt; 1 end ) { :ok , 1 } iex&gt; { :none , 1 } |&gt; Result . default_as ( :ok , &amp; &amp;1 ) { :ok , 1 } iex&gt; :ok |&gt; Result . default_as ( :ok , &quot;hello&quot; ) :ok iex&gt; { :ok , 1 } |&gt; Result . default_as ( :ok , &quot;hello&quot; ) { :ok , 1 } iex&gt; { :ok , 1 , 2 } |&gt; Result . default_as ( :ok , &quot;hello&quot; ) { :ok , 1 , 2 } iex&gt; { :anything , 1 } |&gt; Result . default_as ( :ok , &quot;hello&quot; ) { :anything , 1 } iex&gt; &quot;bare value&quot; |&gt; Result . default_as ( :ok , &quot;hello&quot; ) &quot;bare value&quot;","ref":"OkThen.Result.html#default_as/3","title":"OkThen.Result.default_as/3","type":"function"},{"doc":"If result is tagged :none , returns func_or_value wrapped as an :error result. Otherwise, returns result . If func_or_value is a function, the returned value is used as the new value. If the new value is nil , then the result will remain :none . Consider using none_then/2 if you don't want this behaviour. If result is not tagged :none , result is returned as-is. Equivalent to default_as(result, :error, func_or_value) . See default_as/3 . Examples iex&gt; :none |&gt; Result . default_error ( &quot;hello&quot; ) { :error , &quot;hello&quot; } iex&gt; :none |&gt; Result . default_error ( { } ) :error iex&gt; :none |&gt; Result . default_error ( nil ) :none iex&gt; :none |&gt; Result . default_error ( fn -&gt; 1 end ) { :error , 1 } iex&gt; :none |&gt; Result . default_error ( fn { } -&gt; 1 end ) { :error , 1 } iex&gt; { :none , 1 } |&gt; Result . default_error ( &amp; &amp;1 ) { :error , 1 } iex&gt; :error |&gt; Result . default_error ( &quot;hello&quot; ) :error iex&gt; { :error , 1 } |&gt; Result . default_error ( &quot;hello&quot; ) { :error , 1 } iex&gt; { :error , 1 , 2 } |&gt; Result . default_error ( &quot;hello&quot; ) { :error , 1 , 2 } iex&gt; { :anything , 1 } |&gt; Result . default_error ( &quot;hello&quot; ) { :anything , 1 } iex&gt; &quot;bare value&quot; |&gt; Result . default_error ( &quot;hello&quot; ) &quot;bare value&quot;","ref":"OkThen.Result.html#default_error/2","title":"OkThen.Result.default_error/2","type":"function"},{"doc":"If result is tagged with :error , passes the wrapped value into the provided function (if provided) and returns :none . If result is not tagged with :error , result is returned as-is. Equivalent to tagged_consume(result, :error, func_or_value) . See tagged_consume/3 . Examples iex&gt; :error |&gt; Result . error_consume ( ) :none iex&gt; :error |&gt; Result . error_consume ( fn -&gt; &quot;hello&quot; end ) :none iex&gt; :error |&gt; Result . error_consume ( fn { } -&gt; &quot;hello&quot; end ) :none iex&gt; { :error , 1 } |&gt; Result . error_consume ( fn 1 -&gt; &quot;hello&quot; end ) :none iex&gt; { :error , 1 , 2 } |&gt; Result . error_consume ( fn 1 , 2 -&gt; &quot;hello&quot; end ) ** (ArgumentError) Value-mapping function must have arity between 0 and 1. iex&gt; { :error , 1 , 2 } |&gt; Result . error_consume ( fn { 1 , 2 } -&gt; &quot;hello&quot; end ) :none iex&gt; :ok |&gt; Result . error_consume ( fn { } -&gt; &quot;hello&quot; end ) :ok iex&gt; { :ok , 1 } |&gt; Result . error_consume ( fn 1 -&gt; &quot;hello&quot; end ) { :ok , 1 } iex&gt; &quot;bare value&quot; |&gt; Result . error_consume ( ) &quot;bare value&quot;","ref":"OkThen.Result.html#error_consume/2","title":"OkThen.Result.error_consume/2","type":"function"},{"doc":"If result is tagged :error , transforms the wrapped value by passing it into the provided mapping function, and replacing it with the returned value. If func_or_value is not a function, then it is used directly as the new value. If the new value would be nil , then :none is returned as the result instead. Consider piping into |&gt; none_then({:error, nil}) if you really want {:error, nil} . See none_then/2 . If result is not tagged :error , result is returned as-is. Equivalent to tagged_map(result, :error, func_or_value) . See tagged_map/3 . Examples iex&gt; :error |&gt; Result . error_map ( &quot;hello&quot; ) { :error , &quot;hello&quot; } iex&gt; { :error , 1 } |&gt; Result . error_map ( &quot;hello&quot; ) { :error , &quot;hello&quot; } iex&gt; { :error , 1 } |&gt; Result . error_map ( nil ) :none iex&gt; :none |&gt; Result . error_map ( &quot;hello&quot; ) :none iex&gt; :error |&gt; Result . error_map ( fn { } -&gt; &quot;hello&quot; end ) { :error , &quot;hello&quot; } iex&gt; { :error , 1 } |&gt; Result . error_map ( fn 1 -&gt; &quot;hello&quot; end ) { :error , &quot;hello&quot; } iex&gt; { :error , 1 , 2 } |&gt; Result . error_map ( fn { 1 , 2 } -&gt; &quot;hello&quot; end ) { :error , &quot;hello&quot; } iex&gt; { :error , 1 , 2 } |&gt; Result . error_map ( fn 1 , 2 -&gt; &quot;hello&quot; end ) ** (ArgumentError) Value-mapping function must have arity between 0 and 1. iex&gt; { :error , 1 , 2 } |&gt; Result . error_map ( fn { 1 , 2 } -&gt; { } end ) :error iex&gt; :ok |&gt; Result . error_map ( fn _ -&gt; &quot;hello&quot; end ) :ok iex&gt; { :ok , 1 } |&gt; Result . error_map ( fn _ -&gt; &quot;hello&quot; end ) { :ok , 1 } iex&gt; { :ok , 1 , 2 } |&gt; Result . error_map ( fn _ -&gt; &quot;hello&quot; end ) { :ok , 1 , 2 } iex&gt; :none |&gt; Result . error_map ( fn _ -&gt; &quot;hello&quot; end ) :none iex&gt; :something_else |&gt; Result . error_map ( fn _ -&gt; &quot;hello&quot; end ) :something_else iex&gt; &quot;bare value&quot; |&gt; Result . error_map ( fn _ -&gt; &quot;hello&quot; end ) &quot;bare value&quot; iex&gt; &quot;bare value&quot; |&gt; Result . error_map ( &quot;hello&quot; ) &quot;bare value&quot;","ref":"OkThen.Result.html#error_map/2","title":"OkThen.Result.error_map/2","type":"function"},{"doc":"If result is not tagged with :error , passes the tag and wrapped value into the provided function and returns the result. If the function has arity 1, then only the wrapped value is passed in. If func_or_value is not a function, then it is used directly as the new value. If result is tagged with :error , result is returned as-is. Use this function in a pipeline to branch the happy path into another function, or as a kind of case expression to handle multiple types of result without the boilerplate of copying through an error result. Be aware that no attempt is made to ensure the return value from the function is a tagged tuple. However, all functions are tolerant of untagged results, and on input will interpret them as an {:untagged, value} tuple. Equivalent to tagged_or_else(result, :error, func_or_value) . See tagged_or_else/3 . Examples iex&gt; :ok ...&gt; |&gt; Result . error_or_else ( fn ...&gt; :ok , { } -&gt; { :ok , &quot;hello&quot; } ...&gt; end ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 } ...&gt; |&gt; Result . error_or_else ( fn ...&gt; :ok , 1 -&gt; { :ok , &quot;hello&quot; } ...&gt; end ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 , 2 } ...&gt; |&gt; Result . error_or_else ( fn ...&gt; :ok , { 1 , 2 } -&gt; { :ok , &quot;matched ok&quot; } ...&gt; :none , { } -&gt; { :ok , &quot;matched none&quot; } ...&gt; end ) { :ok , &quot;matched ok&quot; } iex&gt; :none ...&gt; |&gt; Result . error_or_else ( fn ...&gt; :ok , { 1 , 2 } -&gt; { :ok , &quot;matched ok&quot; } ...&gt; :none , { } -&gt; { :ok , &quot;matched none&quot; } ...&gt; end ) { :ok , &quot;matched none&quot; } iex&gt; { :error , &quot;just error&quot; } ...&gt; |&gt; Result . error_or_else ( fn ...&gt; :ok , { 1 , 2 } -&gt; { :ok , &quot;matched ok&quot; } ...&gt; :none , { } -&gt; { :ok , &quot;matched none&quot; } ...&gt; end ) { :error , &quot;just error&quot; } iex&gt; { :ok , 1 , 2 } ...&gt; |&gt; Result . error_or_else ( fn ...&gt; { 1 , 2 } -&gt; { :ok , &quot;matched two terms&quot; } ...&gt; 1 -&gt; { :ok , &quot;matched one term&quot; } ...&gt; { } -&gt; { :ok , &quot;matched no terms&quot; } ...&gt; end ) { :ok , &quot;matched two terms&quot; } iex&gt; { :ok , 1 } ...&gt; |&gt; Result . error_or_else ( fn ...&gt; { 1 , 2 } -&gt; { :ok , &quot;matched two terms&quot; } ...&gt; 1 -&gt; { :ok , &quot;matched one term&quot; } ...&gt; { } -&gt; { :ok , &quot;matched no terms&quot; } ...&gt; end ) { :ok , &quot;matched one term&quot; } iex&gt; :ok ...&gt; |&gt; Result . error_or_else ( fn ...&gt; { 1 , 2 } -&gt; { :ok , &quot;matched two terms&quot; } ...&gt; 1 -&gt; { :ok , &quot;matched one term&quot; } ...&gt; { } -&gt; { :ok , &quot;matched no terms&quot; } ...&gt; end ) { :ok , &quot;matched no terms&quot; } iex&gt; :none ...&gt; |&gt; Result . error_or_else ( fn ...&gt; { 1 , 2 } -&gt; { :ok , &quot;matched two terms&quot; } ...&gt; 1 -&gt; { :ok , &quot;matched one term&quot; } ...&gt; { } -&gt; { :ok , &quot;matched no terms&quot; } ...&gt; end ) { :ok , &quot;matched no terms&quot; } iex&gt; :ok |&gt; Result . error_or_else ( { :ok , &quot;hello&quot; } ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 } |&gt; Result . error_or_else ( { :ok , &quot;hello&quot; } ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 } |&gt; Result . error_or_else ( &quot;bare value&quot; ) &quot;bare value&quot; iex&gt; &quot;bare value&quot; |&gt; Result . error_or_else ( fn _ -&gt; :none end ) :none iex&gt; &quot;bare value&quot; |&gt; Result . error_or_else ( fn ...&gt; :untagged , &quot;bare value&quot; -&gt; :none ...&gt; end ) :none","ref":"OkThen.Result.html#error_or_else/2","title":"OkThen.Result.error_or_else/2","type":"function"},{"doc":"If result is tagged :error , replaces the tag with new_tag , returning a new tagged tuple. Equivalent to tagged_retag(result, :error, new_tag) . See tagged_retag/3 . Examples iex&gt; :error |&gt; Result . error_retag ( :none ) :none iex&gt; { :error , &quot;hello&quot; } |&gt; Result . error_retag ( :ok ) { :ok , &quot;hello&quot; } iex&gt; { :error , 1 , 2 } |&gt; Result . error_retag ( :ok ) { :ok , { 1 , 2 } } iex&gt; { :ok , 1 , 2 } |&gt; Result . error_retag ( :error ) { :ok , 1 , 2 } iex&gt; :error |&gt; Result . error_retag ( &quot;string&quot; ) ** (ArgumentError) Expected atom as new tag, got: &quot;string&quot;. iex&gt; &quot;bare value&quot; |&gt; Result . error_retag ( :ok ) &quot;bare value&quot;","ref":"OkThen.Result.html#error_retag/2","title":"OkThen.Result.error_retag/2","type":"function"},{"doc":"If result is tagged :error , passes the wrapped value into func_or_value and returns the result. If a function is not provided, the argument at the same position is returned as-is. If result is not tagged with the specified tag atom, result is returned as-is. Use this function to pipe results into functions that return tagged tuples. Be aware that no attempt is made to ensure the return value from the function is a tagged tuple. However, all functions are tolerant of untagged results, and on input will interpret them as an {:untagged, value} tuple. Equivalent to tagged_then(result, :error, func_or_value) . See tagged_then/3 . Examples iex&gt; :error |&gt; Result . error_then ( { :ok , &quot;hello&quot; } ) { :ok , &quot;hello&quot; } iex&gt; { :error , 1 } |&gt; Result . error_then ( { :ok , &quot;hello&quot; } ) { :ok , &quot;hello&quot; } iex&gt; :none |&gt; Result . error_then ( { :ok , &quot;hello&quot; } ) :none iex&gt; :error |&gt; Result . error_then ( fn -&gt; { :ok , &quot;hello&quot; } end ) { :ok , &quot;hello&quot; } iex&gt; :error |&gt; Result . error_then ( fn { } -&gt; { :ok , &quot;hello&quot; } end ) { :ok , &quot;hello&quot; } iex&gt; { :error , 1 } |&gt; Result . error_then ( fn 1 -&gt; { :ok , &quot;hello&quot; } end ) { :ok , &quot;hello&quot; } iex&gt; { :error , 1 , 2 } |&gt; Result . error_then ( fn { 1 , 2 } -&gt; { :ok , &quot;hello&quot; } end ) { :ok , &quot;hello&quot; } iex&gt; { :error , 1 , 2 } |&gt; Result . error_then ( fn 1 , 2 -&gt; { :ok , &quot;hello&quot; } end ) ** (ArgumentError) Value-mapping function must have arity between 0 and 1. iex&gt; { :error , 1 , 2 } |&gt; Result . error_then ( fn { 1 , 2 } -&gt; :ok end ) :ok iex&gt; :ok |&gt; Result . error_then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) :ok iex&gt; { :ok , 1 } |&gt; Result . error_then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) { :ok , 1 } iex&gt; { :ok , 1 , 2 } |&gt; Result . error_then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) { :ok , 1 , 2 } iex&gt; :none |&gt; Result . error_then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) :none iex&gt; :something_else |&gt; Result . error_then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) :something_else iex&gt; &quot;bare value&quot; |&gt; Result . error_then ( { :ok , &quot;hello&quot; } ) &quot;bare value&quot; iex&gt; &quot;bare value&quot; |&gt; Result . error_then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) &quot;bare value&quot;","ref":"OkThen.Result.html#error_then/2","title":"OkThen.Result.error_then/2","type":"function"},{"doc":"Same as error_unwrap_or_else/2 , except raises ArgumentError if result is not tagged :error . Examples iex&gt; { :error , &quot;hello&quot; } |&gt; Result . error_unwrap! ( ) &quot;hello&quot; iex&gt; :error |&gt; Result . error_unwrap! ( ) { } iex&gt; :ok |&gt; Result . error_unwrap! ( ) ** (ArgumentError) Value is not tagged error: :ok. iex&gt; { :ok , &quot;hello&quot; } |&gt; Result . error_unwrap! ( ) ** (ArgumentError) Value is not tagged error: {:ok, &quot;hello&quot;}. iex&gt; :none |&gt; Result . error_unwrap! ( ) ** (ArgumentError) Value is not tagged error: :none. iex&gt; &quot;hello&quot; |&gt; Result . error_unwrap! ( ) ** (ArgumentError) Value is not tagged error: {:untagged, &quot;hello&quot;}.","ref":"OkThen.Result.html#error_unwrap!/1","title":"OkThen.Result.error_unwrap!/1","type":"function"},{"doc":"Returns the wrapped value if result is tagged :error . Otherwise, passes the tag and wrapped value into the provided function and returns the result. If the function has arity 1, then only the wrapped value is passed in. An arity-0 function is also accepted. If func_or_value is not a function, then it is used directly as the new value. See also tagged_or_else/3 . Equivalent to tagged_unwrap_or_else(result, :error, default) . See tagged_unwrap_or_else/3 . Examples iex&gt; { :error , &quot;hello&quot; } |&gt; Result . error_unwrap_or_else ( &quot;default&quot; ) &quot;hello&quot; iex&gt; :error |&gt; Result . error_unwrap_or_else ( &quot;default&quot; ) { } iex&gt; :ok |&gt; Result . error_unwrap_or_else ( &quot;default&quot; ) &quot;default&quot; iex&gt; { :ok , &quot;hello&quot; } |&gt; Result . error_unwrap_or_else ( &quot;default&quot; ) &quot;default&quot; iex&gt; { :ok , &quot;hello&quot; } ...&gt; |&gt; Result . error_unwrap_or_else ( fn ...&gt; :ok , &quot;hello&quot; -&gt; &quot;default&quot; ...&gt; end ) &quot;default&quot; iex&gt; { :ok , &quot;hello&quot; } ...&gt; |&gt; Result . error_unwrap_or_else ( fn ...&gt; &quot;hello&quot; -&gt; &quot;default&quot; ...&gt; end ) &quot;default&quot; iex&gt; { :error , &quot;hello&quot; } |&gt; Result . tagged_unwrap_or_else ( :ok , fn -&gt; &quot;default&quot; end ) &quot;default&quot; iex&gt; :none |&gt; Result . error_unwrap_or_else ( &quot;default&quot; ) &quot;default&quot; iex&gt; &quot;hello&quot; |&gt; Result . error_unwrap_or_else ( &quot;default&quot; ) &quot;default&quot;","ref":"OkThen.Result.html#error_unwrap_or_else/2","title":"OkThen.Result.error_unwrap_or_else/2","type":"function"},{"doc":"Converts value into a maybe(v) result: {:ok, value} | :none If value is nil , then the result will be :none . See also from!/1 . Otherwise, the result will be a two-element tuple, where the first element is :ok , and the second element is value . Examples iex&gt; Result . from ( &quot;hello&quot; ) { :ok , &quot;hello&quot; } iex&gt; Result . from ( { 1 , 2 } ) { :ok , { 1 , 2 } } iex&gt; Result . from ( { } ) :ok iex&gt; Result . from ( nil ) :none","ref":"OkThen.Result.html#from/1","title":"OkThen.Result.from/1","type":"function"},{"doc":"Same as from/1 , except raises ArgumentError if value is nil . Examples iex&gt; Result . from! ( &quot;hello&quot; ) { :ok , &quot;hello&quot; } iex&gt; Result . from! ( { 1 , 2 } ) { :ok , { 1 , 2 } } iex&gt; Result . from! ( nil ) ** (ArgumentError) Value is nil.","ref":"OkThen.Result.html#from!/1","title":"OkThen.Result.from!/1","type":"function"},{"doc":"Converts value into a maybe_is(tag) result: {atom(), any()} | :none If value is nil , then the result will be :none . See also from_as!/2 . Otherwise, the result will be a two-element tuple, where the first element is the provided tag, and the second element is value . Examples iex&gt; &quot;hello&quot; |&gt; Result . from_as ( :ok ) { :ok , &quot;hello&quot; } iex&gt; Result . from_as ( { 1 , 2 } , :something ) { :something , { 1 , 2 } } iex&gt; Result . from_as ( { } , :any_atom ) :any_atom iex&gt; Result . from_as ( nil , :ok ) :none","ref":"OkThen.Result.html#from_as/2","title":"OkThen.Result.from_as/2","type":"function"},{"doc":"Same as from_as/2 , except raises ArgumentError if value is nil . Examples iex&gt; &quot;hello&quot; |&gt; Result . from_as! ( :ok ) { :ok , &quot;hello&quot; } iex&gt; nil |&gt; Result . from_as! ( :ok ) ** (ArgumentError) Value is nil.","ref":"OkThen.Result.html#from_as!/2","title":"OkThen.Result.from_as!/2","type":"function"},{"doc":"Converts value into a maybe_error(e) result: {:error, value} | :none If value is nil , then the result will be :none . See also from_error!/1 . Otherwise, the result will be a two-element tuple, where the first element is :error , and the second element is value . Examples iex&gt; Result . from_error ( &quot;hello&quot; ) { :error , &quot;hello&quot; } iex&gt; Result . from_error ( { 1 , 2 } ) { :error , { 1 , 2 } } iex&gt; Result . from_error ( { } ) :error iex&gt; Result . from_error ( nil ) :none","ref":"OkThen.Result.html#from_error/1","title":"OkThen.Result.from_error/1","type":"function"},{"doc":"Same as from_error/1 , except raises ArgumentError if value is nil . Examples iex&gt; Result . from_error! ( &quot;hello&quot; ) { :error , &quot;hello&quot; } iex&gt; Result . from_error! ( { 1 , 2 } ) { :error , { 1 , 2 } } iex&gt; Result . from_error! ( nil ) ** (ArgumentError) Value is nil.","ref":"OkThen.Result.html#from_error!/1","title":"OkThen.Result.from_error!/1","type":"function"},{"doc":"Returns true if result is tagged :error . Equivalent to is_tagged(value, :error) . See is_tagged/2 . Examples iex&gt; Result . is_error ( :error ) true iex&gt; Result . is_error ( { :error , &quot;hello&quot; } ) true iex&gt; Result . is_error ( { :ok , &quot;hello&quot; } ) false","ref":"OkThen.Result.html#is_error/1","title":"OkThen.Result.is_error/1","type":"macro"},{"doc":"Returns true if result is tagged :none . Equivalent to is_tagged(value, :none) . See is_tagged/2 . Examples iex&gt; Result . is_none ( :none ) true iex&gt; Result . is_none ( { :ok , &quot;hello&quot; } ) false iex&gt; Result . is_none ( { :error , &quot;hello&quot; } ) false","ref":"OkThen.Result.html#is_none/1","title":"OkThen.Result.is_none/1","type":"macro"},{"doc":"Returns true if result is tagged :ok . Equivalent to is_tagged(value, :ok) . See is_tagged/2 . Examples iex&gt; Result . is_ok ( :ok ) true iex&gt; Result . is_ok ( { :ok , &quot;hello&quot; } ) true iex&gt; Result . is_ok ( { :error , &quot;hello&quot; } ) false","ref":"OkThen.Result.html#is_ok/1","title":"OkThen.Result.is_ok/1","type":"macro"},{"doc":"Returns true if result is tagged with the specified tag atom. Examples iex&gt; Result . is_tagged ( :ok , :ok ) true iex&gt; Result . is_tagged ( { :ok , &quot;hello&quot; } , :ok ) true iex&gt; Result . is_tagged ( { :error , &quot;hello&quot; } , :ok ) false iex&gt; Result . is_tagged ( { :ok , 1 , 2 } , :ok ) true iex&gt; Result . is_tagged ( { :ok , { 1 , 2 } } , :ok ) true iex&gt; Result . is_tagged ( { :strange , &quot;hello&quot; } , :strange ) true iex&gt; hello = fn -&gt; &quot;hello&quot; end ...&gt; hello . ( ) |&gt; Result . is_tagged ( :ok ) false","ref":"OkThen.Result.html#is_tagged/2","title":"OkThen.Result.is_tagged/2","type":"macro"},{"doc":"Returns true if result is a tagged tuple. Examples iex&gt; Result . is_tagged_tuple ( :ok ) true iex&gt; Result . is_tagged_tuple ( { :ok , &quot;hello&quot; } ) true iex&gt; Result . is_tagged_tuple ( { :error , &quot;hello&quot; } ) true iex&gt; Result . is_tagged_tuple ( { :ok , 1 , 2 } ) true iex&gt; Result . is_tagged_tuple ( { :ok , { 1 , 2 } } ) true iex&gt; Result . is_tagged_tuple ( { :strange , &quot;hello&quot; } ) true iex&gt; Result . is_tagged_tuple ( { &quot;ok&quot; , &quot;hello&quot; } ) false iex&gt; func = fn -&gt; &quot;hello&quot; end ...&gt; func . ( ) |&gt; Result . is_tagged_tuple ( ) false iex&gt; func = fn -&gt; nil end ...&gt; func . ( ) |&gt; Result . is_tagged_tuple ( ) false iex&gt; Result . is_tagged_tuple ( { nil , &quot;hello&quot; } ) false","ref":"OkThen.Result.html#is_tagged_tuple/1","title":"OkThen.Result.is_tagged_tuple/1","type":"macro"},{"doc":"If result is tagged :ok , transforms the wrapped value by passing it into the provided mapping function, and replacing it with the returned value. If func_or_value is not a function, then it is used directly as the new value. If the new value would be nil , then :none is returned as the result instead. Consider piping into |&gt; none_then({:ok, nil}) if you really want {:ok, nil} . See none_then/2 . If result is not tagged :ok , result is returned as-is. Equivalent to tagged_map(result, :ok, func_or_value) . See tagged_map/3 . Examples iex&gt; :ok |&gt; Result . map ( &quot;hello&quot; ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 } |&gt; Result . map ( &quot;hello&quot; ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 } |&gt; Result . map ( nil ) :none iex&gt; :none |&gt; Result . map ( &quot;hello&quot; ) :none iex&gt; :ok |&gt; Result . map ( fn { } -&gt; &quot;hello&quot; end ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 } |&gt; Result . map ( fn 1 -&gt; &quot;hello&quot; end ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 , 2 } |&gt; Result . map ( fn { 1 , 2 } -&gt; &quot;hello&quot; end ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 , 2 } |&gt; Result . map ( fn 1 , 2 -&gt; &quot;hello&quot; end ) ** (ArgumentError) Value-mapping function must have arity between 0 and 1. iex&gt; { :ok , 1 , 2 } |&gt; Result . map ( fn { 1 , 2 } -&gt; { } end ) :ok iex&gt; :error |&gt; Result . map ( fn _ -&gt; &quot;hello&quot; end ) :error iex&gt; { :error , 1 } |&gt; Result . map ( fn _ -&gt; &quot;hello&quot; end ) { :error , 1 } iex&gt; { :error , 1 , 2 } |&gt; Result . map ( fn _ -&gt; &quot;hello&quot; end ) { :error , 1 , 2 } iex&gt; :none |&gt; Result . map ( fn _ -&gt; &quot;hello&quot; end ) :none iex&gt; :something_else |&gt; Result . map ( fn _ -&gt; &quot;hello&quot; end ) :something_else iex&gt; &quot;bare value&quot; |&gt; Result . map ( fn _ -&gt; &quot;hello&quot; end ) &quot;bare value&quot; iex&gt; &quot;bare value&quot; |&gt; Result . map ( &quot;hello&quot; ) &quot;bare value&quot;","ref":"OkThen.Result.html#map/2","title":"OkThen.Result.map/2","type":"function"},{"doc":"If result is tagged :none , replaces the tag with new_tag . Equivalent to tagged_retag(result, :none, new_tag) . See tagged_retag/3 . Examples iex&gt; :none |&gt; Result . none_retag ( :ok ) :ok iex&gt; :error |&gt; Result . none_retag ( :ok ) :error iex&gt; :ok |&gt; Result . none_retag ( :ok ) :ok iex&gt; :none |&gt; Result . none_retag ( &quot;string&quot; ) ** (ArgumentError) Expected atom as new tag, got: &quot;string&quot;. iex&gt; &quot;bare value&quot; |&gt; Result . none_retag ( :error ) &quot;bare value&quot;","ref":"OkThen.Result.html#none_retag/2","title":"OkThen.Result.none_retag/2","type":"function"},{"doc":"If result is tagged :none , calls func_or_value and returns the result. If func_or_value is not a function, then it is returned as-is. If result is not tagged with the specified tag atom, result is returned as-is. Use this function to pipe results into functions that return tagged tuples. Be aware that no attempt is made to ensure the return value from the function is a tagged tuple. However, all functions are tolerant of untagged results, and on input will interpret them as an {:untagged, value} tuple. Equivalent to tagged_then(result, :none, func_or_value) . See tagged_then/3 . Examples iex&gt; :none |&gt; Result . none_then ( :ok ) :ok iex&gt; :none |&gt; Result . none_then ( fn { } -&gt; { :ok , &quot;hello&quot; } end ) { :ok , &quot;hello&quot; } iex&gt; :error |&gt; Result . none_then ( fn -&gt; { :ok , &quot;hello&quot; } end ) :error iex&gt; :error |&gt; Result . none_then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) :error iex&gt; { :error , 1 } |&gt; Result . none_then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) { :error , 1 } iex&gt; { :error , 1 , 2 } |&gt; Result . none_then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) { :error , 1 , 2 } iex&gt; :something_else |&gt; Result . none_then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) :something_else iex&gt; &quot;bare value&quot; |&gt; Result . none_then ( :ok ) &quot;bare value&quot; iex&gt; &quot;bare value&quot; |&gt; Result . none_then ( fn _ -&gt; :ok end ) &quot;bare value&quot;","ref":"OkThen.Result.html#none_then/2","title":"OkThen.Result.none_then/2","type":"function"},{"doc":"Converts result from a variety of accepted result-like terms into an atom or a two-element tagged tuple. If result is not a tagged tuple, it is wrapped as a new result Examples iex&gt; Result . normalize ( :ok ) :ok iex&gt; Result . normalize ( { :ok , &quot;hello&quot; } ) { :ok , &quot;hello&quot; } iex&gt; Result . normalize ( { :ok , 1 , 2 } ) { :ok , { 1 , 2 } } iex&gt; Result . normalize ( :error ) :error iex&gt; Result . normalize ( { :error , &quot;hello&quot; } ) { :error , &quot;hello&quot; } iex&gt; Result . normalize ( { :error , 1 , 2 } ) { :error , { 1 , 2 } } iex&gt; Result . normalize ( :none ) :none iex&gt; Result . normalize ( { :strange , [ &quot;hello&quot; , 1 , 2 ] } ) { :strange , [ &quot;hello&quot; , 1 , 2 ] } iex&gt; Result . normalize ( &quot;hello&quot; ) { :untagged , &quot;hello&quot; } iex&gt; Result . normalize ( { 1 , 2 } ) { :untagged , { 1 , 2 } } iex&gt; Result . normalize ( { } ) :untagged iex&gt; Result . normalize ( { 1 , 2 } , :error ) { :error , { 1 , 2 } } iex&gt; Result . normalize ( nil ) :none iex&gt; Result . normalize ( nil , :error ) :none","ref":"OkThen.Result.html#normalize/2","title":"OkThen.Result.normalize/2","type":"function"},{"doc":"Same as normalize/1 , except raises ArgumentError if value is untagged. Examples iex&gt; Result . normalize! ( &quot;hello&quot; ) ** (ArgumentError) Result is untagged: &quot;hello&quot;","ref":"OkThen.Result.html#normalize!/1","title":"OkThen.Result.normalize!/1","type":"function"},{"doc":"If result is not tagged with :ok , passes the tag and wrapped value into the provided function and returns the result. If the function has arity 1, then only the wrapped value is passed in. If func_or_value is not a function, then it is used directly as the new value. If result is tagged with :ok , result is returned as-is. Use this function in a pipeline to branch the unhappy path into another function, or as a kind of case expression to handle multiple types of result without the boilerplate of copying through a successful result. Be aware that no attempt is made to ensure the return value from the function is a tagged tuple. However, all functions are tolerant of untagged results, and on input will interpret them as an {:untagged, value} tuple. Equivalent to tagged_or_else(result, :ok, func_or_value) . See tagged_or_else/3 . Examples iex&gt; :error ...&gt; |&gt; Result . or_else ( fn ...&gt; :error , { } -&gt; { :ok , &quot;hello&quot; } ...&gt; end ) { :ok , &quot;hello&quot; } iex&gt; { :error , 1 } ...&gt; |&gt; Result . or_else ( fn ...&gt; :error , 1 -&gt; { :ok , &quot;hello&quot; } ...&gt; end ) { :ok , &quot;hello&quot; } iex&gt; { :error , 1 , 2 } ...&gt; |&gt; Result . or_else ( fn ...&gt; :error , { 1 , 2 } -&gt; { :ok , &quot;matched error&quot; } ...&gt; :none , { } -&gt; { :ok , &quot;matched none&quot; } ...&gt; end ) { :ok , &quot;matched error&quot; } iex&gt; :none ...&gt; |&gt; Result . or_else ( fn ...&gt; :error , { 1 , 2 } -&gt; { :ok , &quot;matched error&quot; } ...&gt; :none , { } -&gt; { :ok , &quot;matched none&quot; } ...&gt; end ) { :ok , &quot;matched none&quot; } iex&gt; { :ok , &quot;just ok&quot; } ...&gt; |&gt; Result . or_else ( fn ...&gt; :error , { 1 , 2 } -&gt; { :error , &quot;matched error&quot; } ...&gt; :none , { } -&gt; { :ok , &quot;matched none&quot; } ...&gt; end ) { :ok , &quot;just ok&quot; } iex&gt; { :error , 1 , 2 } ...&gt; |&gt; Result . or_else ( fn ...&gt; { 1 , 2 } -&gt; { :ok , &quot;matched two terms&quot; } ...&gt; 1 -&gt; { :ok , &quot;matched one term&quot; } ...&gt; { } -&gt; { :ok , &quot;matched no terms&quot; } ...&gt; end ) { :ok , &quot;matched two terms&quot; } iex&gt; { :error , 1 } ...&gt; |&gt; Result . or_else ( fn ...&gt; { 1 , 2 } -&gt; { :ok , &quot;matched two terms&quot; } ...&gt; 1 -&gt; { :ok , &quot;matched one term&quot; } ...&gt; { } -&gt; { :ok , &quot;matched no terms&quot; } ...&gt; end ) { :ok , &quot;matched one term&quot; } iex&gt; :error ...&gt; |&gt; Result . or_else ( fn ...&gt; { 1 , 2 } -&gt; { :ok , &quot;matched two terms&quot; } ...&gt; 1 -&gt; { :ok , &quot;matched one term&quot; } ...&gt; { } -&gt; { :ok , &quot;matched no terms&quot; } ...&gt; end ) { :ok , &quot;matched no terms&quot; } iex&gt; :none ...&gt; |&gt; Result . or_else ( fn ...&gt; { 1 , 2 } -&gt; { :ok , &quot;matched two terms&quot; } ...&gt; 1 -&gt; { :ok , &quot;matched one term&quot; } ...&gt; { } -&gt; { :ok , &quot;matched no terms&quot; } ...&gt; end ) { :ok , &quot;matched no terms&quot; } iex&gt; :error |&gt; Result . or_else ( { :ok , &quot;hello&quot; } ) { :ok , &quot;hello&quot; } iex&gt; { :error , 1 } |&gt; Result . or_else ( { :ok , &quot;hello&quot; } ) { :ok , &quot;hello&quot; } iex&gt; { :error , 1 } |&gt; Result . or_else ( &quot;bare value&quot; ) &quot;bare value&quot; iex&gt; &quot;bare value&quot; |&gt; Result . or_else ( fn _ -&gt; :none end ) :none iex&gt; &quot;bare value&quot; |&gt; Result . or_else ( fn ...&gt; :untagged , &quot;bare value&quot; -&gt; :none ...&gt; end ) :none","ref":"OkThen.Result.html#or_else/2","title":"OkThen.Result.or_else/2","type":"function"},{"doc":"If result is tagged :ok , replaces the tag with new_tag , returning a new tagged tuple. Equivalent to tagged_retag(result, :ok, new_tag) . See tagged_retag/3 . Examples iex&gt; :ok |&gt; Result . retag ( :none ) :none iex&gt; { :ok , &quot;hello&quot; } |&gt; Result . retag ( :error ) { :error , &quot;hello&quot; } iex&gt; { :ok , 1 , 2 } |&gt; Result . retag ( :error ) { :error , { 1 , 2 } } iex&gt; { :error , 1 , 2 } |&gt; Result . retag ( :ok ) { :error , 1 , 2 } iex&gt; :ok |&gt; Result . retag ( &quot;string&quot; ) ** (ArgumentError) Expected atom as new tag, got: &quot;string&quot;. iex&gt; &quot;bare value&quot; |&gt; Result . error_retag ( :error ) &quot;bare value&quot;","ref":"OkThen.Result.html#retag/2","title":"OkThen.Result.retag/2","type":"function"},{"doc":"If result is tagged with the specified tag atom, passes the wrapped value into the provided function (if provided) and returns :none . If result is not tagged with the specified tag atom, result is returned as-is. Examples iex&gt; :ok |&gt; Result . tagged_consume ( :ok ) :none iex&gt; :error |&gt; Result . tagged_consume ( :error , fn -&gt; &quot;hello&quot; end ) :none iex&gt; :error |&gt; Result . tagged_consume ( :error , fn { } -&gt; &quot;hello&quot; end ) :none iex&gt; { :some , 1 } |&gt; Result . tagged_consume ( :some , fn 1 -&gt; &quot;hello&quot; end ) :none iex&gt; { :ok , 1 , 2 } |&gt; Result . tagged_consume ( :ok , fn 1 , 2 -&gt; &quot;hello&quot; end ) ** (ArgumentError) Value-mapping function must have arity between 0 and 1. iex&gt; { :ok , 1 , 2 } |&gt; Result . tagged_consume ( :ok , fn { 1 , 2 } -&gt; &quot;hello&quot; end ) :none iex&gt; :error |&gt; Result . tagged_consume ( :ok , fn { } -&gt; &quot;hello&quot; end ) :error iex&gt; { :error , 1 } |&gt; Result . tagged_consume ( :ok , fn 1 -&gt; &quot;hello&quot; end ) { :error , 1 } iex&gt; &quot;bare value&quot; |&gt; Result . tagged_consume ( :ok ) &quot;bare value&quot;","ref":"OkThen.Result.html#tagged_consume/3","title":"OkThen.Result.tagged_consume/3","type":"function"},{"doc":"If result is tagged with the specified tag atom, transforms the wrapped value by passing it into the provided mapping function, and replacing it with the returned value. If a function is not provided, the argument at the same position is used as the new value. If the new value would be nil , then :none is returned as the result instead. Consider piping into |&gt; none_then({tag, nil}) if you really want {tag, nil} . See none_then/2 . If result is not tagged with the specified tag atom, result is returned as-is. Examples iex&gt; :ok |&gt; Result . tagged_map ( :ok , &quot;hello&quot; ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 } |&gt; Result . tagged_map ( :ok , &quot;hello&quot; ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 } |&gt; Result . tagged_map ( :ok , nil ) :none iex&gt; :none |&gt; Result . tagged_map ( :ok , &quot;hello&quot; ) :none iex&gt; :ok |&gt; Result . tagged_map ( :ok , fn -&gt; &quot;hello&quot; end ) { :ok , &quot;hello&quot; } iex&gt; :ok |&gt; Result . tagged_map ( :ok , fn { } -&gt; &quot;hello&quot; end ) { :ok , &quot;hello&quot; } iex&gt; { :bla , 1 } |&gt; Result . tagged_map ( :bla , fn 1 -&gt; &quot;hello&quot; end ) { :bla , &quot;hello&quot; } iex&gt; { :some , 1 , 2 } |&gt; Result . tagged_map ( :some , fn { 1 , 2 } -&gt; &quot;hello&quot; end ) { :some , &quot;hello&quot; } iex&gt; { :ok , 1 , 2 } |&gt; Result . tagged_map ( :ok , fn 1 , 2 -&gt; &quot;hello&quot; end ) ** (ArgumentError) Value-mapping function must have arity between 0 and 1. iex&gt; { :ok , 1 , 2 } |&gt; Result . tagged_map ( :ok , fn { 1 , 2 } -&gt; { } end ) :ok iex&gt; :error |&gt; Result . tagged_map ( :ok , fn _ -&gt; &quot;hello&quot; end ) :error iex&gt; { :error , 1 } |&gt; Result . tagged_map ( :ok , fn _ -&gt; &quot;hello&quot; end ) { :error , 1 } iex&gt; { :error , 1 , 2 } |&gt; Result . tagged_map ( :ok , fn _ -&gt; &quot;hello&quot; end ) { :error , 1 , 2 } iex&gt; :none |&gt; Result . tagged_map ( :ok , fn _ -&gt; &quot;hello&quot; end ) :none iex&gt; :something_else |&gt; Result . tagged_map ( :ok , fn _ -&gt; &quot;hello&quot; end ) :something_else iex&gt; &quot;bare value&quot; |&gt; Result . tagged_map ( :ok , fn _ -&gt; &quot;hello&quot; end ) &quot;bare value&quot; iex&gt; &quot;bare value&quot; |&gt; Result . tagged_map ( :untagged , fn _ -&gt; &quot;hello&quot; end ) { :untagged , &quot;hello&quot; } iex&gt; &quot;bare value&quot; |&gt; Result . tagged_map ( :ok , &quot;hello&quot; ) &quot;bare value&quot; iex&gt; &quot;bare value&quot; |&gt; Result . tagged_map ( :untagged , &quot;hello&quot; ) { :untagged , &quot;hello&quot; }","ref":"OkThen.Result.html#tagged_map/3","title":"OkThen.Result.tagged_map/3","type":"function"},{"doc":"If result is not tagged with the specified tag atom, passes the tag and wrapped value into the provided function and returns the result. If the function has arity 1, then only the wrapped value is passed in. An arity-0 function is also accepted. If func_or_value is not a function, then it is used directly as the new value. If result is tagged with the specified tag atom, result is returned as-is. Use this function in a pipeline to branch away from the happy path, or as a kind of case expression to handle multiple types of result without the boilerplate of copying through a successful result. Be aware that no attempt is made to ensure the return value from the function is a tagged tuple. However, all functions are tolerant of untagged results, and on input will interpret them as an {:untagged, value} tuple. Examples iex&gt; :error ...&gt; |&gt; Result . tagged_or_else ( :ok , fn ...&gt; :error , { } -&gt; { :ok , &quot;hello&quot; } ...&gt; end ) { :ok , &quot;hello&quot; } iex&gt; { :error , 1 } ...&gt; |&gt; Result . tagged_or_else ( :ok , fn ...&gt; :error , 1 -&gt; { :ok , &quot;hello&quot; } ...&gt; end ) { :ok , &quot;hello&quot; } iex&gt; { :error , 1 , 2 } ...&gt; |&gt; Result . tagged_or_else ( :ok , fn ...&gt; :error , { 1 , 2 } -&gt; { :ok , &quot;matched error&quot; } ...&gt; :none , { } -&gt; { :ok , &quot;matched none&quot; } ...&gt; end ) { :ok , &quot;matched error&quot; } iex&gt; :none ...&gt; |&gt; Result . tagged_or_else ( :ok , fn ...&gt; :error , { 1 , 2 } -&gt; { :ok , &quot;matched error&quot; } ...&gt; :none , { } -&gt; { :ok , &quot;matched none&quot; } ...&gt; end ) { :ok , &quot;matched none&quot; } iex&gt; { :ok , &quot;just ok&quot; } ...&gt; |&gt; Result . tagged_or_else ( :ok , fn ...&gt; :error , { 1 , 2 } -&gt; { :ok , &quot;matched error&quot; } ...&gt; :none , { } -&gt; { :ok , &quot;matched none&quot; } ...&gt; end ) { :ok , &quot;just ok&quot; } iex&gt; { :error , 1 , 2 } ...&gt; |&gt; Result . tagged_or_else ( :ok , fn ...&gt; { 1 , 2 } -&gt; { :ok , &quot;matched two terms&quot; } ...&gt; 1 -&gt; { :ok , &quot;matched one term&quot; } ...&gt; { } -&gt; { :ok , &quot;matched no terms&quot; } ...&gt; end ) { :ok , &quot;matched two terms&quot; } iex&gt; { :error , 1 } ...&gt; |&gt; Result . tagged_or_else ( :ok , fn ...&gt; { 1 , 2 } -&gt; { :ok , &quot;matched two terms&quot; } ...&gt; 1 -&gt; { :ok , &quot;matched one term&quot; } ...&gt; { } -&gt; { :ok , &quot;matched no terms&quot; } ...&gt; end ) { :ok , &quot;matched one term&quot; } iex&gt; :error ...&gt; |&gt; Result . tagged_or_else ( :ok , fn ...&gt; { 1 , 2 } -&gt; { :ok , &quot;matched two terms&quot; } ...&gt; 1 -&gt; { :ok , &quot;matched one term&quot; } ...&gt; { } -&gt; { :ok , &quot;matched no terms&quot; } ...&gt; end ) { :ok , &quot;matched no terms&quot; } iex&gt; :none ...&gt; |&gt; Result . tagged_or_else ( :ok , fn ...&gt; { 1 , 2 } -&gt; { :ok , &quot;matched two terms&quot; } ...&gt; 1 -&gt; { :ok , &quot;matched one term&quot; } ...&gt; { } -&gt; { :ok , &quot;matched no terms&quot; } ...&gt; end ) { :ok , &quot;matched no terms&quot; } iex&gt; :none ...&gt; |&gt; Result . tagged_or_else ( :ok , fn -&gt; { :ok , &quot;catch-all value&quot; } end ) { :ok , &quot;catch-all value&quot; } iex&gt; :error |&gt; Result . tagged_or_else ( :ok , { :ok , &quot;hello&quot; } ) { :ok , &quot;hello&quot; } iex&gt; { :error , 1 } |&gt; Result . tagged_or_else ( :ok , { :ok , &quot;hello&quot; } ) { :ok , &quot;hello&quot; } iex&gt; { :error , 1 } |&gt; Result . tagged_or_else ( :ok , &quot;bare value&quot; ) &quot;bare value&quot; iex&gt; &quot;bare value&quot; |&gt; Result . tagged_or_else ( :ok , fn _ -&gt; :none end ) :none iex&gt; &quot;bare value&quot; |&gt; Result . tagged_or_else ( :ok , fn ...&gt; :untagged , &quot;bare value&quot; -&gt; :none ...&gt; end ) :none iex&gt; &quot;bare value&quot; |&gt; Result . tagged_or_else ( :untagged , fn _ -&gt; :none end ) &quot;bare value&quot;","ref":"OkThen.Result.html#tagged_or_else/3","title":"OkThen.Result.tagged_or_else/3","type":"function"},{"doc":"If result is tagged with the specified tag atom, replaces the tag with new_tag , returning a new tagged tuple. Examples iex&gt; :ok |&gt; Result . tagged_retag ( :ok , :none ) :none iex&gt; { :ok , &quot;hello&quot; } |&gt; Result . tagged_retag ( :ok , :error ) { :error , &quot;hello&quot; } iex&gt; { :error , 1 , 2 } |&gt; Result . tagged_retag ( :error , :ok ) { :ok , { 1 , 2 } } iex&gt; { :ok , 1 , 2 } |&gt; Result . tagged_retag ( :error , :ok ) { :ok , 1 , 2 } iex&gt; :ok |&gt; Result . tagged_retag ( :ok , &quot;string&quot; ) ** (ArgumentError) Expected atom as new tag, got: &quot;string&quot;. iex&gt; &quot;bare value&quot; |&gt; Result . tagged_retag ( :ok , :error ) &quot;bare value&quot; iex&gt; &quot;bare value&quot; |&gt; Result . tagged_retag ( :untagged , :error ) { :error , &quot;bare value&quot; }","ref":"OkThen.Result.html#tagged_retag/3","title":"OkThen.Result.tagged_retag/3","type":"function"},{"doc":"If result is tagged with the specified tag atom, passes the wrapped value into the provided function and returns the result. If func_or_value is not a function, then it is returned as-is. If result is not tagged with the specified tag atom, result is returned as-is. Use this function to pipe results into functions that return tagged tuples. Be aware that no attempt is made to ensure the return value from the function is a tagged tuple. However, all functions are tolerant of untagged results, and on input will interpret them as an {:untagged, value} tuple. Examples iex&gt; :ok |&gt; Result . tagged_then ( :ok , { :ok , &quot;hello&quot; } ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 } |&gt; Result . tagged_then ( :ok , { :ok , &quot;hello&quot; } ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 } |&gt; Result . tagged_then ( :ok , &quot;bare value&quot; ) &quot;bare value&quot; iex&gt; :none |&gt; Result . tagged_then ( :ok , { :ok , &quot;hello&quot; } ) :none iex&gt; :ok |&gt; Result . tagged_then ( :ok , fn -&gt; &quot;bare value&quot; end ) &quot;bare value&quot; iex&gt; :ok |&gt; Result . tagged_then ( :ok , fn { } -&gt; &quot;bare value&quot; end ) &quot;bare value&quot; iex&gt; :ok |&gt; Result . tagged_then ( :ok , fn { } -&gt; { :ok , &quot;hello&quot; } end ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 } |&gt; Result . tagged_then ( :ok , fn 1 -&gt; { :ok , &quot;hello&quot; } end ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 , 2 } |&gt; Result . tagged_then ( :ok , fn { 1 , 2 } -&gt; { :ok , &quot;hello&quot; } end ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 , 2 } |&gt; Result . tagged_then ( :ok , fn 1 , 2 -&gt; { :ok , &quot;hello&quot; } end ) ** (ArgumentError) Value-mapping function must have arity between 0 and 1. iex&gt; { :ok , 1 , 2 } |&gt; Result . tagged_then ( :ok , fn { 1 , 2 } -&gt; { :ok , { } } end ) { :ok , { } } iex&gt; :error |&gt; Result . tagged_then ( :ok , fn _ -&gt; { :ok , &quot;hello&quot; } end ) :error iex&gt; { :error , 1 } |&gt; Result . tagged_then ( :ok , fn _ -&gt; { :ok , &quot;hello&quot; } end ) { :error , 1 } iex&gt; { :error , 1 , 2 } |&gt; Result . tagged_then ( :ok , fn _ -&gt; { :ok , &quot;hello&quot; } end ) { :error , 1 , 2 } iex&gt; :none |&gt; Result . tagged_then ( :ok , fn _ -&gt; { :ok , &quot;hello&quot; } end ) :none iex&gt; :something_else |&gt; Result . tagged_then ( :ok , fn _ -&gt; { :ok , &quot;hello&quot; } end ) :something_else iex&gt; &quot;bare value&quot; |&gt; Result . tagged_then ( :ok , fn _ -&gt; :none end ) &quot;bare value&quot; iex&gt; &quot;bare value&quot; |&gt; Result . tagged_then ( :untagged , fn _ -&gt; :none end ) :none iex&gt; &quot;bare value&quot; |&gt; Result . tagged_then ( :ok , :none ) &quot;bare value&quot; iex&gt; &quot;bare value&quot; |&gt; Result . tagged_then ( :untagged , :none ) :none","ref":"OkThen.Result.html#tagged_then/3","title":"OkThen.Result.tagged_then/3","type":"function"},{"doc":"Same as tagged_unwrap_or_else/3 , except raises ArgumentError if result is not tagged with the specified tag atom. Examples iex&gt; { :ok , &quot;hello&quot; } |&gt; Result . tagged_unwrap! ( :ok ) &quot;hello&quot; iex&gt; :some |&gt; Result . tagged_unwrap! ( :some ) { } iex&gt; :error |&gt; Result . tagged_unwrap! ( :ok ) ** (ArgumentError) Value is not tagged ok: :error. iex&gt; { :ok , &quot;hello&quot; } |&gt; Result . tagged_unwrap! ( :error ) ** (ArgumentError) Value is not tagged error: {:ok, &quot;hello&quot;}. iex&gt; :none |&gt; Result . tagged_unwrap! ( :ok ) ** (ArgumentError) Value is not tagged ok: :none. iex&gt; &quot;hello&quot; |&gt; Result . tagged_unwrap! ( :untagged ) &quot;hello&quot; iex&gt; &quot;hello&quot; |&gt; Result . tagged_unwrap! ( :ok ) ** (ArgumentError) Value is not tagged ok: {:untagged, &quot;hello&quot;}.","ref":"OkThen.Result.html#tagged_unwrap!/2","title":"OkThen.Result.tagged_unwrap!/2","type":"function"},{"doc":"Returns the wrapped value if result is tagged with the specified tag atom. Otherwise, passes the tag and wrapped value into the provided function and returns the result. If the function has arity 1, then only the wrapped value is passed in. An arity-0 function is also accepted. If func_or_value is not a function, then it is used directly as the new value. See also tagged_or_else/3 . Examples iex&gt; { :ok , &quot;hello&quot; } |&gt; Result . tagged_unwrap_or_else ( :ok , &quot;default&quot; ) &quot;hello&quot; iex&gt; :some |&gt; Result . tagged_unwrap_or_else ( :some , &quot;default&quot; ) { } iex&gt; :error |&gt; Result . tagged_unwrap_or_else ( :ok , &quot;default&quot; ) &quot;default&quot; iex&gt; { :error , &quot;hello&quot; } |&gt; Result . tagged_unwrap_or_else ( :ok , &quot;default&quot; ) &quot;default&quot; iex&gt; { :error , &quot;hello&quot; } ...&gt; |&gt; Result . tagged_unwrap_or_else ( :ok , fn ...&gt; :error , &quot;hello&quot; -&gt; &quot;default&quot; ...&gt; end ) &quot;default&quot; iex&gt; { :error , &quot;hello&quot; } ...&gt; |&gt; Result . tagged_unwrap_or_else ( :ok , fn ...&gt; &quot;hello&quot; -&gt; &quot;default&quot; ...&gt; end ) &quot;default&quot; iex&gt; { :error , &quot;hello&quot; } |&gt; Result . tagged_unwrap_or_else ( :ok , fn -&gt; &quot;default&quot; end ) &quot;default&quot; iex&gt; :none |&gt; Result . tagged_unwrap_or_else ( :ok , &quot;default&quot; ) &quot;default&quot; iex&gt; &quot;hello&quot; |&gt; Result . tagged_unwrap_or_else ( :ok , &quot;default&quot; ) &quot;default&quot; iex&gt; &quot;hello&quot; |&gt; Result . tagged_unwrap_or_else ( :untagged , &quot;default&quot; ) &quot;hello&quot;","ref":"OkThen.Result.html#tagged_unwrap_or_else/3","title":"OkThen.Result.tagged_unwrap_or_else/3","type":"function"},{"doc":"If result is tagged :ok , passes the wrapped value into func_or_value and returns the result. If a function is not provided, the argument at the same position is returned as-is. If result is not tagged with the specified tag atom, result is returned as-is. Use this function to pipe results into functions that return tagged tuples. Be aware that no attempt is made to ensure the return value from the function is a tagged tuple. However, all functions are tolerant of untagged results, and on input will interpret them as an {:untagged, value} tuple. Equivalent to tagged_then(result, :ok, func_or_value) . See tagged_then/3 . Examples iex&gt; :ok |&gt; Result . then ( { :ok , &quot;hello&quot; } ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 } |&gt; Result . then ( { :ok , &quot;hello&quot; } ) { :ok , &quot;hello&quot; } iex&gt; :none |&gt; Result . then ( { :ok , &quot;hello&quot; } ) :none iex&gt; :ok |&gt; Result . then ( fn { } -&gt; { :ok , &quot;hello&quot; } end ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 } |&gt; Result . then ( fn 1 -&gt; { :ok , &quot;hello&quot; } end ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 , 2 } |&gt; Result . then ( fn { 1 , 2 } -&gt; { :ok , &quot;hello&quot; } end ) { :ok , &quot;hello&quot; } iex&gt; { :ok , 1 , 2 } |&gt; Result . then ( fn 1 , 2 -&gt; { :ok , &quot;hello&quot; } end ) ** (ArgumentError) Value-mapping function must have arity between 0 and 1. iex&gt; { :ok , 1 , 2 } |&gt; Result . then ( fn { 1 , 2 } -&gt; :ok end ) :ok iex&gt; :error |&gt; Result . then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) :error iex&gt; { :error , 1 } |&gt; Result . then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) { :error , 1 } iex&gt; { :error , 1 , 2 } |&gt; Result . then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) { :error , 1 , 2 } iex&gt; :none |&gt; Result . then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) :none iex&gt; :something_else |&gt; Result . then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) :something_else iex&gt; &quot;bare value&quot; |&gt; Result . then ( { :ok , &quot;hello&quot; } ) &quot;bare value&quot; iex&gt; &quot;bare value&quot; |&gt; Result . then ( fn _ -&gt; { :ok , &quot;hello&quot; } end ) &quot;bare value&quot;","ref":"OkThen.Result.html#then/2","title":"OkThen.Result.then/2","type":"function"},{"doc":"Same as unwrap_or_else/2 , except raises ArgumentError if result is not tagged :ok . Examples iex&gt; { :ok , &quot;hello&quot; } |&gt; Result . unwrap! ( ) &quot;hello&quot; iex&gt; :ok |&gt; Result . unwrap! ( ) { } iex&gt; :error |&gt; Result . unwrap! ( ) ** (ArgumentError) Value is not tagged ok: :error. iex&gt; { :error , &quot;hello&quot; } |&gt; Result . unwrap! ( ) ** (ArgumentError) Value is not tagged ok: {:error, &quot;hello&quot;}. iex&gt; :none |&gt; Result . unwrap! ( ) ** (ArgumentError) Value is not tagged ok: :none. iex&gt; &quot;hello&quot; |&gt; Result . unwrap! ( ) ** (ArgumentError) Value is not tagged ok: {:untagged, &quot;hello&quot;}.","ref":"OkThen.Result.html#unwrap!/1","title":"OkThen.Result.unwrap!/1","type":"function"},{"doc":"Returns the wrapped value if result is tagged :ok . Otherwise, passes the tag and wrapped value into the provided function and returns the result. If the function has arity 1, then only the wrapped value is passed in. An arity-0 function is also accepted. If func_or_value is not a function, then it is used directly as the new value. See also tagged_or_else/3 . Equivalent to tagged_unwrap_or_else(result, :ok, default) . See tagged_unwrap_or_else/3 . Examples iex&gt; { :ok , &quot;hello&quot; } |&gt; Result . unwrap_or_else ( &quot;default&quot; ) &quot;hello&quot; iex&gt; :ok |&gt; Result . unwrap_or_else ( &quot;default&quot; ) { } iex&gt; :error |&gt; Result . unwrap_or_else ( &quot;default&quot; ) &quot;default&quot; iex&gt; { :error , &quot;hello&quot; } |&gt; Result . unwrap_or_else ( &quot;default&quot; ) &quot;default&quot; iex&gt; { :error , &quot;hello&quot; } ...&gt; |&gt; Result . unwrap_or_else ( fn ...&gt; :error , &quot;hello&quot; -&gt; &quot;default&quot; ...&gt; end ) &quot;default&quot; iex&gt; { :error , &quot;hello&quot; } ...&gt; |&gt; Result . unwrap_or_else ( fn ...&gt; &quot;hello&quot; -&gt; &quot;default&quot; ...&gt; end ) &quot;default&quot; iex&gt; { :error , &quot;hello&quot; } |&gt; Result . unwrap_or_else ( fn -&gt; &quot;default&quot; end ) &quot;default&quot; iex&gt; :none |&gt; Result . unwrap_or_else ( &quot;default&quot; ) &quot;default&quot; iex&gt; &quot;hello&quot; |&gt; Result . unwrap_or_else ( &quot;default&quot; ) &quot;default&quot;","ref":"OkThen.Result.html#unwrap_or_else/2","title":"OkThen.Result.unwrap_or_else/2","type":"function"},{"doc":"","ref":"OkThen.Result.html#t:error/1","title":"OkThen.Result.error/1","type":"type"},{"doc":"","ref":"OkThen.Result.html#t:maybe/0","title":"OkThen.Result.maybe/0","type":"type"},{"doc":"","ref":"OkThen.Result.html#t:maybe/1","title":"OkThen.Result.maybe/1","type":"type"},{"doc":"","ref":"OkThen.Result.html#t:maybe/2","title":"OkThen.Result.maybe/2","type":"type"},{"doc":"","ref":"OkThen.Result.html#t:maybe_error/0","title":"OkThen.Result.maybe_error/0","type":"type"},{"doc":"","ref":"OkThen.Result.html#t:maybe_error/1","title":"OkThen.Result.maybe_error/1","type":"type"},{"doc":"","ref":"OkThen.Result.html#t:maybe_is/1","title":"OkThen.Result.maybe_is/1","type":"type"},{"doc":"","ref":"OkThen.Result.html#t:maybe_is/2","title":"OkThen.Result.maybe_is/2","type":"type"},{"doc":"","ref":"OkThen.Result.html#t:ok/1","title":"OkThen.Result.ok/1","type":"type"},{"doc":"","ref":"OkThen.Result.html#t:ok_or/1","title":"OkThen.Result.ok_or/1","type":"type"},{"doc":"","ref":"OkThen.Result.html#t:ok_or/2","title":"OkThen.Result.ok_or/2","type":"type"},{"doc":"","ref":"OkThen.Result.html#t:result_input/0","title":"OkThen.Result.result_input/0","type":"type"},{"doc":"","ref":"OkThen.Result.html#t:tagged/0","title":"OkThen.Result.tagged/0","type":"type"},{"doc":"Functions for processing tagged tuples inside Enums.","ref":"OkThen.Result.Enum.html","title":"OkThen.Result.Enum","type":"module"},{"doc":"Collects an Enum of results into a single result. If all results were tagged :ok , then a result will be returned tagged with :ok , whose value is a list of the wrapped values from each element in the list. Otherwise, the result whose tag didn't match tag is returned. Equivalent to collect_tagged(results, :ok) . See collect_tagged/2 . Examples iex&gt; [ :ok , :ok ] ...&gt; |&gt; Result.Enum . collect ( ) { :ok , [ { } , { } ] } iex&gt; [ :ok , :ok , :ok , :error , { :error , 2 } ] ...&gt; |&gt; Result.Enum . collect ( ) { :error , { } } iex&gt; [ { :ok , 1 } , { :ok , 1 , 2 } , :ok ] ...&gt; |&gt; Result.Enum . collect ( ) { :ok , [ 1 , { 1 , 2 } , { } ] } iex&gt; [ { :ok , 1 } , { :ok , 1 , 2 } , { :something , 1 } , :ok ] ...&gt; |&gt; Result.Enum . collect ( ) { :something , 1 }","ref":"OkThen.Result.Enum.html#collect/1","title":"OkThen.Result.Enum.collect/1","type":"function"},{"doc":"Collects an Enum of results into a single result. If all results were tagged with the specified tag , then a result will be returned tagged with tag , whose value is a list of the wrapped values from each element in the list. Otherwise, the result whose tag didn't match tag is returned. Examples iex&gt; [ :ok , :ok ] ...&gt; |&gt; Result.Enum . collect_tagged ( :ok ) { :ok , [ { } , { } ] } iex&gt; [ :ok , :ok , :ok , :error , { :error , 2 } ] ...&gt; |&gt; Result.Enum . collect_tagged ( :ok ) { :error , { } } iex&gt; [ { :ok , 1 } , { :ok , 1 , 2 } , :ok ] ...&gt; |&gt; Result.Enum . collect_tagged ( :ok ) { :ok , [ 1 , { 1 , 2 } , { } ] } iex&gt; [ { :ok , 1 } , { :ok , 1 , 2 } , { :something , 1 } , :ok ] ...&gt; |&gt; Result.Enum . collect_tagged ( :ok ) { :something , 1 }","ref":"OkThen.Result.Enum.html#collect_tagged/2","title":"OkThen.Result.Enum.collect_tagged/2","type":"function"},{"doc":"Collects an Enum of results into a map, with result values grouped by their tag. Examples iex&gt; [ :ok , :ok , :ok , :error , :error ] ...&gt; |&gt; Result.Enum . group_by_tag ( ) %{ error : [ { } , { } ] , ok : [ { } , { } , { } ] } iex&gt; [ { :ok , 1 } , { :ok , 2 } , { :ok , 3 } , { :error , 4 } , { :error , 5 } ] ...&gt; |&gt; Result.Enum . group_by_tag ( ) %{ error : [ 4 , 5 ] , ok : [ 1 , 2 , 3 ] } iex&gt; [ { :ok , 1 } , { :ok , 2 , 3 } , :none , { :error , 4 } , { :another , 5 } ] ...&gt; |&gt; Result.Enum . group_by_tag ( ) %{ another : [ 5 ] , error : [ 4 ] , none : [ { } ] , ok : [ 1 , { 2 , 3 } ] }","ref":"OkThen.Result.Enum.html#group_by_tag/1","title":"OkThen.Result.Enum.group_by_tag/1","type":"function"},{"doc":"Convenience operators for handling of tagged results in pipelines.","ref":"OkThen.Result.Pipe.html","title":"OkThen.Result.Pipe","type":"module"},{"doc":"Equivalent to OkThen.Result.map(result, func_or_default) . See OkThen.Result.map/2 . Examples iex&gt; add_one = &amp; ( &amp;1 + 1 ) iex&gt; { :ok , 1 } ...&gt; ~&gt; add_one . ( ) ...&gt; ~&gt; add_one . ( ) { :ok , 3 } iex&gt; add_one = &amp; ( &amp;1 + 1 ) iex&gt; { :error , 1 } ...&gt; ~&gt; add_one . ( ) ...&gt; ~&gt; add_one . ( ) { :error , 1 } iex&gt; { :ok , 1 } ...&gt; ~&gt; &quot;hello&quot; { :ok , &quot;hello&quot; } iex&gt; { :error , 1 } ...&gt; ~&gt; &quot;hello&quot; { :error , 1 } iex&gt; &quot;bare value&quot; ...&gt; ~&gt; &quot;hello&quot; &quot;bare value&quot; iex&gt; add_one = &amp; ( &amp;1 + 1 ) iex&gt; &quot;bare value&quot; ...&gt; ~&gt; add_one . ( ) &quot;bare value&quot;","ref":"OkThen.Result.Pipe.html#~%3E/2","title":"OkThen.Result.Pipe.~>/2","type":"macro"},{"doc":"Equivalent to OkThen.Result.then(result, func_or_default) . See OkThen.Result.then/2 . Examples iex&gt; add_one = &amp; ( { :ok , &amp;1 + 1 } ) iex&gt; { :ok , 1 } ...&gt; ~&gt;&gt; add_one . ( ) ...&gt; ~&gt;&gt; add_one . ( ) { :ok , 3 } iex&gt; add_one = &amp; ( { :ok , &amp;1 + 1 } ) iex&gt; { :error , 1 } ...&gt; ~&gt;&gt; add_one . ( ) ...&gt; ~&gt;&gt; add_one . ( ) { :error , 1 } iex&gt; { :ok , 1 } ...&gt; ~&gt;&gt; { :ok , &quot;hello&quot; } { :ok , &quot;hello&quot; } iex&gt; { :error , 1 } ...&gt; ~&gt;&gt; { :ok , &quot;hello&quot; } { :error , 1 } iex&gt; &quot;bare value&quot; ...&gt; ~&gt;&gt; { :ok , &quot;hello&quot; } &quot;bare value&quot; iex&gt; add_one = &amp; ( &amp;1 + 1 ) iex&gt; &quot;bare value&quot; ...&gt; ~&gt;&gt; add_one . ( ) &quot;bare value&quot;","ref":"OkThen.Result.Pipe.html#~%3E%3E/2","title":"OkThen.Result.Pipe.~>>/2","type":"macro"},{"doc":"OK then... The Swiss Army Knife for tagged tuple pipelines Looking for a better way to handle errors , optional results , and default values ? Wish there were a really consistent and full-featured API to handle tagged tuples in pipelines ? How to install Why does this exist? API Documentation","ref":"readme.html","title":"OK then...","type":"extras"},{"doc":"Wrap values: Result . from ( &quot;hello&quot; ) # {:ok, &quot;hello&quot;} Result . from ( 1 ) # {:ok, 1} Result . from ( nil ) # :none Result . from_error ( 1 ) # {:error, 1} Result . from_as ( 1 , :some ) # {:some, 1} Map values selectively by tag: def pipeline ( value_r ) do value_r |&gt; Result . map ( &amp; &amp;1 * 2 ) |&gt; Result . error_map ( &amp; { :bad_input_value , &amp;1 } ) |&gt; Result . tagged_map ( :add_2 , &amp; &amp;1 + 2 ) end Result . from ( 1 ) |&gt; pipeline ( ) # {:ok, 2} Result . from ( nil ) |&gt; pipeline ( ) # :none Result . from_error ( 1 ) |&gt; pipeline ( ) # {:error, {:bad_input_value, 1}} Result . from_as ( 1 , :add_2 ) |&gt; pipeline ( ) # {:add_2, 3} Or apply functions selectively by tag: def double ( value ) , do : { :ok , value * 2 } def error ( value ) , do : { :error , value } { :ok , 1 } |&gt; Result . then ( &amp; double / 1 ) |&gt; Result . then ( &amp; double / 1 ) # {:ok, 4} { :ok , 1 } |&gt; Result . then ( &amp; error / 1 ) |&gt; Result . then ( &amp; double / 1 ) # {:error, 1} { :ok , 1 } |&gt; Result . then ( &amp; double / 1 ) |&gt; Result . then ( &amp; error / 1 ) # {:error, 2} { :ok , 1 } |&gt; Result . then ( &amp; error / 1 ) |&gt; Result . error_then ( &amp; double / 1 ) # {:ok, 2} And handle unexpected values safely: def to_nil ( _value ) , do : nil def error ( value ) , do : { :error , value } def unwrap_result ( result ) do Result . default ( &quot;default&quot; ) |&gt; Result . unwrap_or_else ( &quot;failsafe&quot; ) end Result . from ( 1 ) # {:ok, 1} |&gt; unwrap_result ( ) # 1 Result . from ( 1 ) # {:ok, 1} |&gt; Result . map ( &amp; to_nil / 1 ) # :none |&gt; unwrap_result ( ) # &quot;default&quot; Result . from ( 1 ) # {:ok, 1} |&gt; Result . then ( &amp; error / 1 ) # {:error, 1} |&gt; unwrap_result ( ) # &quot;failsafe&quot; Typespecs: @spec a ( ) :: Result . ok_or ( any ( ) ) @spec a ( ) :: :ok | { :error , any ( ) } @spec b ( ) :: Result . ok_or ( integer ( ) , any ( ) ) @spec b ( ) :: { :ok , integer ( ) } | { :error , any ( ) } @spec c ( ) :: Result . maybe ( integer ( ) ) @spec c ( ) :: { :ok , integer ( ) } | :none @spec d ( ) :: Result . maybe ( integer ( ) , any ( ) ) @spec d ( ) :: { :ok , integer ( ) } | :none | { :error , any ( ) } You can even handle tagged tuples inside Enums: # Tags other than :ok and :error are supported too :) [ { :ok , 1 } , { :ok , 2 } , { :ok , 3 } , { :error , 4 } , { :error , 5 } ] |&gt; Result.Enum . group_by_tag ( ) %{ error : [ 4 , 5 ] , ok : [ 1 , 2 , 3 ] } [ { :ok , 1 } , { :ok , 2 } , { :ok , 3 } ] |&gt; Result.Enum . collect ( ) { :ok , [ 1 , 2 , 3 ] } [ { :ok , 1 } , { :ok , 2 } , { :error , 3 } , { :ok , 4 } ] |&gt; Result.Enum . collect ( ) { :error , 3 } Check out the API documentation for a full list of supported functions, guards, and types.","ref":"readme.html#at-a-glance","title":"OK then... - At a glance","type":"extras"},{"doc":"Because: Remembering to check for nil is the bane of any programmer's life. It pops up everywhere . Tagged tuples , the idiomatic solution to this problem, can become verbose , especially when they need to be passed to several functions, or through a pipeline. Although {:ok, value} | {:error, reason} is ubiquitous, there is no standardised pattern to represent optional values , other than value | nil . Failing to address point 3 leads to code that either : Returns {:ok, nil} , which brings us right back to unexpected nils popping up in the most obscure ways: ** (UndefinedFunctionError) function nil.my_map_key/0 is undefined. Returns {:error, :not_found} or similar, which is often semantically questionable: missing values are often not actually errors . This can lead to confusion in determining how best to handle fallback values and error logging. One solution (adopted by languages such as Rust), is to provide a return type that is explicitly optional . In Elixir we could represent this with an orthogonal type of tagged tuple: { :some , value } | :none The main drawback of this approach is how verbose the tuples can become in log output, especially when nested. { :ok , { :some , % MyStruct { key : &quot;value&quot; } } } { :ok , :none } { :error , { :some , &quot;Example Error&quot; } } { :error , :none } # Is this an error, or a lack of error? To mitigate this issue, the approach taken by this package is to combine the &quot;ok/error&quot; and &quot;some/none&quot; types into a single type of tagged tuple called a &quot;maybe&quot; : { :ok , value } | :none | { :error , reason } Specific functions are provided to handle tagged tuples with these tags ( :ok, :none, :error ). Generic functions also exist to handle any tag, but are slightly less convenient. Functions that create or map tagged tuples will catch nil values and transform the returned result into :none . This time, we can handle an unexpected nil far more elegantly. Here is a slightly contrived example: get_my_map ( ) # nil |&gt; Result . from ( ) # :none |&gt; Result . default ( %{ } ) # {:ok, %{}} |&gt; Result . map ( &amp; Map . get ( :my_map_key ) ) # :none |&gt; Result . unwrap_or_else ( &quot;default&quot; ) # &quot;default&quot; And now imagine we could introduce an unexpected error. This is not altered by default , which only affects results tagged :none . However, unwrap_or_else will catch any result that is not :ok : { :error , &quot;Unexpected Error&quot; } |&gt; Result . default ( %{ } ) # {:error, &quot;Unexpected Error&quot;} |&gt; Result . map ( &amp; Map . get ( :my_map_key ) ) # {:error, &quot;Unexpected Error&quot;} |&gt; Result . unwrap_or_else ( &quot;default&quot; ) # &quot;default&quot; Even better, we could &quot;consume&quot; the error by logging it, then handle the missing value just like before: { :error , &quot;Unexpected Error&quot; } |&gt; Result . error_consume ( &amp; Logger . error / 1 ) # :none (&quot;Unexpected Error&quot; is logged) |&gt; Result . default ( %{ } ) # {:ok, %{}} |&gt; Result . map ( &amp; Map . get ( :my_map_key ) ) # :none |&gt; Result . unwrap! ( ) # &quot;default&quot; Take a look at some more examples .","ref":"readme.html#why","title":"OK then... - Why?","type":"extras"},{"doc":"Simply add the package to your deps in mix.exs : def deps do [ { :ok_then , &quot;~&gt; x.x.x&quot; } # Check &quot;hex&quot; badge at the top for current version ] end","ref":"readme.html#installation","title":"OK then... - Installation","type":"extras"}]